# Plugin for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2000-2003 Andrea Sterbini, a.sterbini@flashnet.it
# Copyright (C) 2001-2006 Peter Thoeny, peter@thoeny.org
# and TWiki Contributors. All Rights Reserved. TWiki Contributors
# are listed in the AUTHORS file in the root of this distribution.
# NOTE: Please extend that file, not this notice.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# For licensing info read LICENSE file in the TWiki root.

#
# Plugin scans attachments before attaching them to the page with clamav virusscanner. 
#

package TWiki::Plugins::ClamAVPlugin;
use Data::Dumper;
use strict;

use ClamAV::Client;
use IO::File;

use vars qw( $VERSION $RELEASE $SHORTDESCRIPTION $debug $pluginName $NO_PREFS_IN_TOPIC $clamavsocket $clamavhost $clamavport);
$VERSION = '$Rev: 0 (12 May 2007) $';
$RELEASE = 'Dakar';
$SHORTDESCRIPTION = 'ClamAVplugin scans attachments and reports viruses.';
$NO_PREFS_IN_TOPIC = 1;
$pluginName = 'ClamAVPlugin';

sub load_clamav
{
	# On demand loading
	eval {use ClamAV::Client; use IO::File; };
	# Determing what to use, socket, host port.
	my $scanner;	
	if(defined($clamavhost) and defined($clamavport))
	{
		$scanner = ClamAV::Client->new(
			socket_host => $clamavhost,
			socket_port => $clamavport
		);
	} else {
	
	$scanner  = ClamAV::Client->new (
			socket_name => $clamavsocket
		);
	}
	if(not defined($scanner) or not $scanner->ping())
	{
		TWiki::Func::writeError("$pluginName: ClamAV deamon not online.");
		return;
	}
	return $scanner;

}

sub initPlugin {
    my( $topic, $web, $user, $installWeb ) = @_;

    # check for Plugins.pm versions
    if( $TWiki::Plugins::VERSION < 1.026 ) {
        TWiki::Func::writeWarning( "Version mismatch between $pluginName and Plugins.pm" );
        return 0;
    }

	 $clamavsocket = $TWiki::cfg{Plugins}{ClamAVPlugin}{Socket} || '/var/run/clamav/clamd.sock';
	 $clamavhost = $TWiki::cfg{Plugins}{ClamAVPlugin}{Host};
	 $clamavport = $TWiki::cfg{Plugins}{ClamAVPlugin}{port};
       
	$debug = $TWiki::cfg{Plugins}{ClamAVPlugin}{Debug} || 1;

    return 1;
}

sub beforeAttachmentSaveHandler {
	my( $attrHashRef, $topic, $web ) = @_;
	# Load ClamAV::Client and IO::File
	my $scanner = &load_clamav();
	return, unless $scanner;

	my $handle = new IO::File;	
	# Open File to scan
	open($handle, $attrHashRef->{tmpFilename});
	my $result = $scanner->scan_stream($handle);
	close $handle;

	# If we found something, we delete the file and throw an oops.

	if(defined($result))
	{
		unlink($attrHashRef->{tmpFilename});
		&woops("Virus Found!", "Virus $result detected. Please scan your computer! Upload aborted", $web, $topic);
		return;
	}
	return;
}

sub woops
{

	my ($header, $message, $web, $topic) = @_;
	my @err;
	eval{use Error qw( :try )};
	throw TWiki::OopsException("generic",
		def => "",
		web => $web, 
		topic =>$topic, 
		params => [$header, $message, "", ""]);

	return 0;
}
1;
